package Mason::Plugin::BOKUTINFilters::Filters;
{
  $Mason::Plugin::BOKUTINFilters::Filters::VERSION = '0.132771';
}

use utf8;
use Mason::PluginRole;
use Safe::Isa;

method Clickable {
    require URI::Find;
    my $finder = URI::Find->new(sub {
	my ($uri, $orig_uri) = @_;
	return qq|<a href="$orig_uri" target="_blank">$orig_uri</a>|;
    });

    sub {
	my $text = shift;
	$finder->find(\$text);
	$text;
    };
}

method ClickableEmail {
    require Email::Find::Loose;
    my $finder = Email::Find::Loose->new(sub {
	my ($email, $orig_email) = @_;
	my ($address) = $email->format;
	return qq|<a href="mailto:$address">$orig_email</a>|;
    });

    sub {
	my $text = shift;
	$finder->find(\$text);
	$text;
    };
}

# http://cpansearch.perl.org/src/ABW/Template-Toolkit-2.22/lib/Template/Filters.pm
method HTMLLineBreak () {
    sub {
        my $text = $_[0];
        $text =~ s|(\r?\n)|<br />$1|g;
        return $text;
    };
}

method LESSp () {
    require CSS::LESSp;
    sub {
	join("", CSS::LESSp->parse($_[0]));
    };
}

method RelativeDate {
    sub {
	if ($_[0]->$_isa("DateTime")) {
	    my ($dt) = @_;
	    my $now = container("dtx")->now;
	    my $dur = $now - $dt; # Datetime::Duration object.
	    if ( my $years = $dur->in_units('years') ) {
		return $years."年前";
	    }
	    elsif ( my $months = $dur->in_units('months') ) {
		return $months."ヶ月前";
	    }
	    elsif ( my $days = $dur->in_units('days') ) {
		return $days."日前";
	    }
	    elsif ( my $hours = $dur->in_units('hours') ) {
		return $hours."時間前";
	    }
	    elsif ( my $minutes = $dur->in_units('minutes') ) {
		return $minutes."分前";
	    }
	    else {
		return "たった今！";
	    }
	}

	return $_[0];
    };
}

method Join ($separator, @args) {
    Mason::DynamicFilter->new(
        filter => sub {
	    my $yield = $_[0];
	    local $_;
            join($separator, map { $yield->() } @args);
        }
    );
}

# http://cpansearch.perl.org/src/MIYAGAWA/Template-Plugin-JavaScript-0.02/lib/Template/Plugin/JavaScript.pm
method JS {
    sub {
        local $_ = shift;
        return '' unless defined $_;

        s!\\!\\\\!g;
        s!(['"])!\\$1!g;
        s!\n!\\n!g;
        s!\f!\\f!g;
        s!\r!\\r!g;
        s!\t!\\t!g;
        s!<!\\x3c!g;
        s!>!\\x3e!g;
        s!&!\\x26!g;
        $_;
    };
}

method Trunc ($max) {
    require Text::VisualWidth::PP;
    $max ||= 20;
    sub {
        my $text = $_[0];
        my $width = Text::VisualWidth::PP::width($text);
        if ( $width > $max ) {
            $text = Text::VisualWidth::PP::trim($text, $max);
        }
        return $text;
    };
}

method Elide ($max) {
    require Text::VisualWidth::PP;
    $max ||= 20;
    sub {
        my $text = $_[0];
        my $width = Text::VisualWidth::PP::width($text);
        if ( $width > ($max-3) ) {
            $text = Text::VisualWidth::PP::trim($text, $max-3) . "...";
        }
        return $text;
    };
}

for my $n ( 1 .. 20 ) {
    $n *= 10;
    for my $method (qw(Trunk Elide)) {
	__PACKAGE__->meta->add_method( "$method$n" => sub { $_[0]->$method($n) } );
    }
}

use Encode;
use POSIX qw(strftime);
method Time ($format) {
    $format ||= '%Y-%m-%d(%a) %H:%M:%S';
    require Date::Parse;
    require Time::Piece;
    sub {
        my $some = defined($_[0]) ? $_[0] : return $_[0];

        if (!blessed($some) and $some =~ m/^(\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2})/) {
            # garbage at end of string in strptime: .766834 at /Users/bokutin/perl5/perlbrew/perls/perl-5.16.2/lib/5.16.2/darwin-thread-multi-2level/Time/Piece.pm line 469.
            # などを避けるため、ミリ秒は捨て、マッチした部分だけを評価する。
            return if $1 eq '0000-00-00 00:00:00';
            my $tp = Time::Piece->strptime($1, '%Y-%m-%d %H:%M:%S');
            return decode('utf8', $tp->strftime($format));
        }
	elsif ($some->$_isa("DateTime")) {
            return $some->strftime($format);
	}
	elsif ($some and $some =~ m/^\d+$/) {
            return strftime($format, localtime($some));
	}
        elsif ($some and $some =~ m/^(?:sun|mon|tue|wed|thu|fri|sat)/i) {
            if (my $time = Date::Parse::str2time($some)) {
                return strftime($format, localtime($time));
            }
        }

	$some;
    };
}

method FormatBytes {
    require Number::Bytes::Human;
    sub {
        defined($_[0]) ? Number::Bytes::Human::format_bytes($_[0]) : $_[0];
    };
}

method YMD2Rel {
    my $notes = $self->m->{notes}{_ymd2rel} ||= {};
    require Time::Piece;
    require Time::Seconds;
    sub {
        my ($str) = @_;
        return $str unless $str;
        $str =~ s/^@{[ $notes->{today}     ||=  Time::Piece->localtime->strftime('%Y-%m-%d') ]}/今日/;
        $str =~ s/^@{[ $notes->{yesterday} ||= (Time::Piece->localtime - Time::Seconds::ONE_DAY())->strftime('%Y-%m-%d') ]}/昨日/;
        $str =~ s/^@{[ $notes->{tommorow}  ||= (Time::Piece->localtime + Time::Seconds::ONE_DAY())->strftime('%Y-%m-%d') ]}/明日/;
        $str;
    };
}

use HTML::Escape ();
with 'Mason::Plugin::HTMLFilters::Filters' => { -excludes => [qw(HTML H)] };

method HTML () {
    sub {
        $_[0]->$_isa('Mason::Plugin::BOKUTINFilters::Type::Raw') ? ${$_[0]} : HTML::Escape::escape_html($_[0]);
    };
}
*H = *HTML;

1;

__END__

=pod

=head1 NAME

Mason::Plugin::BOKUTINFilters::Filters

=head1 VERSION

version 0.132771

=head1 AUTHOR

Tomohiro Hosaka <bokutin@bokut.in>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by Tomohiro Hosaka.  No
license is granted to other entities.

=cut
